//---------------------------------------------------------------------------
// TestEstablishment
//---------------------------------------------------------------------------
#if !defined(TestEstablishment_H)
  #define TestEstablishment

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>
#include <cppunit/TestSuite.h>

class clGridBase;
class clTreePopulation;

/**
* This class tests the establishment behavior.  It writes out parameter files
* and uses them as input to perform setup; it then sets up the desired initial
* conditions, causes the behavior to run, then tests its output.  There will be
* two runs.  The first tests favorability calculations and non-substrate
* establishment; the second tests substrate establishment.
 */
class clTestEstablishment : public CppUnit::TestFixture
{

public:

/**
      Tests establishment using the first set of conditions.  It will cause a
   parameter file to be written and then passed to a clSimManager object.  This
   object will read it.  The establishment behavior is run, seedlings counted,
   and the substrate favorability grid examined.
*/
  void TestEstablishment1();

/**
  Tests establishment using the second set of conditions.  It will cause a
parameter file to be written and then passed to a clSimManager object.  This
object will read it.  The establishment behavior is run and seedlings counted.
*/
  void TestEstablishment2();


/**
  Creates a test suite of all the test cases in this object.  I'm putting
  it here so it will be easier to see and add new cases to.

  @return The test suite.
*/

  static CppUnit::Test * suite()
  {
    CppUnit::TestSuite * suite = new CppUnit::TestSuite("TestEstablishment");

    suite->addTest(
         new CppUnit::TestCaller < clTestEstablishment > ("TestEstablishment1", & clTestEstablishment::TestEstablishment1));
    suite->addTest(
         new CppUnit::TestCaller < clTestEstablishment > ("TestEstablishment2", & clTestEstablishment::TestEstablishment2));

    return suite;
  };

private:

  /** Verifies the favorabilities and number of seedlings for TestEstablishment1.
  * @param p_oSubstrateFavGrid Substrate favorability grid.
  * @param p_oPop Tree population. */
  void VerifyXMLFile1(clGridBase * p_oSubstrateFavGrid, clTreePopulation *p_oPop);

  /** Verifies the numbers of seedlings for TestEstablishment2.
  * @param p_oSubstrateFavGrid Substrate favorability grid.
  * @param p_oPop Tree population.*/
  void VerifyXMLFile2(clGridBase * p_oSubstrateFavGrid, clTreePopulation *p_oPop);

  /**
   Writes a parameter file to direct testing using the first set of testing
   conditions.  This contains all tree information and disperse information.
   It also has a tree map so we'll be dealing with a known list of trees.

   @return Filename written.
  */
  char * WriteXMLFile1();

  /**
  * Writes a parameter file to direct testing using the second set of testing
  * conditions.  This contains all tree information and disperse information.
  * It also has a tree map so we'll be dealing with a known list of trees.
  * The establishing species is #6; this is for a bug fix involving grid codes.
  * @return Filename written.
  */
  char * WriteXMLFile2();
};

//---------------------------------------------------------------------------
#endif // TestEstablishment
