//---------------------------------------------------------------------------
// TestNCIMort
//---------------------------------------------------------------------------
#if !defined(TestNCIMort_H)
  #define TestNCIMort_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

class clSimManager;

/**
* Tests the clNCIMort class.
*/
class clTestNCIMort : public CppUnit::TestFixture {

  public:

  /**
  * Tests the clNCIMort class.
  */
  void TestNCIMort();

  protected:

  /**
  * Calls each of the test runs.
  * @param p_oSimManager Sim Manager object to do the runs.
  */
  void TestNormalProcessing(clSimManager *p_oSimManager);

  /**
  * @todo double-check this one.  Error files need storm parameters.
  *
  * Tests error processing for input files.  Tests:
  * <ul>
  * <li>Max. radius of neighbor effects must be > 0</li>
  * <li>Max probability of survival for each species must be 0-1</li>
  * <li>X0  (Size effect mode) for each species must not = 0</li>
  * <li>Xb (Size effect variance) for each species must not = 0</li>
  * <li>Eta for each species for each damage category beyond undamaged must be
  * between 0 and 1</li>
  * <li>Storm Effect parameters for each species for each damage category beyond
  * undamaged must be between 0 and 1</li>
  * </ul>
  *
  * WriteXMLErrorFileX (where X = 1-12) is written and checked to make sure that
  * each produces an error.
  */
  void TestErrorProcessing(clSimManager *p_oSimManager);

  /**
  * Performs a test run of NCI mortality - a Puerto-Rico-like situation.  This
  * run has no shading effect and uses damage effect.
  * WriteXMLFile1() performs setup for a 1-year timestep; then WriteXMLFile2()
  * performs setup for an otherwise identical 3-year timestep.
  *
  * There are 8 target trees, for which 1000 copies of each will be created.
  * Then the remainder left after a timestep will be checked against the
  * testing spreadsheet expected results.
  * @param p_oSimManager Sim Manager object to do the runs.
  */
  void TestRun1(clSimManager *p_oSimManager);

  /**
  * Tests the mortality calculations at two timestep lengths in a BC-like
  * situation.  It uses no storm damage and simplified crowding -
  * no size sensitivity to NCI (g = 0, d = 1).  It does use Shading.
  * WriteXMLFile3() is used to do setup in the first case, for a 1 year
  * timestep.  Mortality is checked for two target trees.  Then WriteXMLFile4()
  * does setup for a 3 year timestep.
  *
  * The first target tree is too far away from the other trees to be affected.
  * It has the lowest possible value of shade and a zero NCI.  The second
  * target tree is surrounded by 4 trees affecting shade and NCI and two which
  * do not.  1000 copies of each are created.  Then the remainder left after a
  * timestep will be checked against the testing spreadsheet expected results.
  * @param p_oSimManager Sim Manager object to do the runs.
  */
  void TestRun2(clSimManager *p_oSimManager);

  /**
  * Performs Test Run 3.  This run has no storm damage, has variability in
  * functions between species, has neighbors below the minimum neighbor DBH,
  * and doesn't use the DBH divisor.
  *
  * WriteXMLFile5() is used to do setup for the first run, for a 1 year
  * timestep.  1000 copies are created for each of 5 target trees and the
  * number left checked after 1 timestep.  Then WriteXMLFile6()
  * does setup for a 3 year timestep and the process is repeated.
  */
  void TestRun3(clSimManager *p_oSimManager);

  /**
  * Performs Test Run 4.  This run tests variability in light behavior
  * requirements, but is otherwise almost exactly like TestRun3.
  *
  * WriteXMLFile7() is used to do setup for the first run, for a 1 year
  * timestep.  1000 copies are created for each of 5 target trees and the
  * number left checked after 1 timestep.  Then WriteXMLFile8() does setup for
  * a 3 year timestep and the process is repeated.
  */
  void TestRun4(clSimManager *p_oSimManager);

  /**
  * Performs a test run of NCI mortality - a Puerto-Rico-like situation.  This
  * run has no shading effect and uses damage effect.  The damage applier is
  * not applied to all species.
  * WriteXMLFile9() performs setup for a 1-year timestep; then WriteXMLFile10()
  * performs setup for an otherwise identical 3-year timestep.
  *
  * There are 8 target trees, for which 1000 copies of each will be created.
  * Then the remainder left after a timestep will be checked against the
  * testing spreadsheet expected results.
  * @param p_oSimManager Sim Manager object to do the runs.
  */
  void TestRun5(clSimManager *p_oSimManager);

  /**
  * Writes a 1-year timestep file.  This contains 4 species, one unused
  * (species 3; this helps catch array-index problems).  NCI mortality
  * is assigned to all saplings and adults.
  *
  * @return Filename written.
  */
  char* WriteXMLFile1();

  /**
  * Writes a 3-year timestep file.  This contains 4 species, one unused
  * (species 3; this helps catch array-index problems).  NCI mortality
  * is assigned to all saplings and adults.
  *
  * @return Filename written.
  */
  char* WriteXMLFile2();

  /**
  * Writes a 1-year timestep file.  NCI mortality is assigned to species 1
  * adults.
  *
  * @return Filename written.
  */
  char* WriteXMLFile3();

  /**
  * Writes a 3-year timestep file.  NCI mortality is assigned to species 1
  * adults.
  *
  * @return Filename written.
  */
  char* WriteXMLFile4();

  /**
  * Writes a 1-year timestep file for Run 3.
  *
  * @return Filename written.
  */
  char* WriteXMLFile5();

  /**
  * Writes a 3-year timestep file for Run 3.
  *
  * @return Filename written.
  */
  char* WriteXMLFile6();

  /**
  * Writes a 1-year timestep file for Run 4.
  *
  * @return Filename written.
  */
  char* WriteXMLFile7();

  /**
  * Writes a 3-year timestep file for Run 4.
  *
  * @return Filename written.
  */
  char* WriteXMLFile8();

  /**
  * Writes a 1-year timestep file for Run 5.
  *
  * @return Filename written.
  */
  char* WriteXMLFile9();

  /**
  * Writes a 3-year timestep file for Run 5.
  *
  * @return Filename written.
  */
  char* WriteXMLFile10();

  /**
  * Writes an error condition where max radius of neighbor effects is < 0.
  *
  * @return Filename written.
  */
  char* WriteXMLErrorFile1();

  /**
  * Writes an error condition where max probability for survival for a species
  * is not between 0 and 1.
  *
  * @return Filename written.
  */
  char* WriteXMLErrorFile2();

  /**
  * Writes an error condition where Xb for a species = 0.
  *
  * @return Filename written.
  */
  char* WriteXMLErrorFile3();

  /**
  * Writes an error condition where X0 for a species = 0.
  *
  * @return Filename written.
  */
  char* WriteXMLErrorFile4();

  /**
  * Writes an error condition where med. damage eta for a species < 0.
  *
  * @return Filename written.
  */
  char* WriteXMLErrorFile5();

  /**
  * Writes an error condition where med. damage eta for a species > 1.
  *
  * @return Filename written.
  */
  char* WriteXMLErrorFile6();

  /**
  * Writes an error condition where full damage eta for a species < 0.
  *
  * @return Filename written.
  */
  char* WriteXMLErrorFile7();

  /**
  * Writes an error condition where full damage eta for a species > 1.
  *
  * @return Filename written.
  */
  char* WriteXMLErrorFile8();

  /**
  * Writes an error condition where med. damage storm effect for a species < 0.
  *
  * @return Filename written.
  */
  char* WriteXMLErrorFile9();

  /**
  * Writes an error condition where med. damage storm effect for a species > 1.
  *
  * @return Filename written.
  */
  char* WriteXMLErrorFile10();

  /**
  * Writes an error condition where full damage storm effect for a species < 0.
  *
  * @return Filename written.
  */
  char* WriteXMLErrorFile11();

  /**
  * Writes an error condition where full damage storm effect for a species > 1.
  *
  * @return Filename written.
  */
  char* WriteXMLErrorFile12();

  /**
  * Writes an error condition where shading is used but there is no light
  * behavior.
  *
  * @return Filename written.
  */
  char* WriteXMLErrorFile13();

  /**
  * Writes common portions of the test parameter files.  Ends with "</trees>".
  * @param oOut File stream to write to.
  */
  void WritePlotAndTrees(std::fstream &oOut);

  /**
  * Writes storm parameters.  Everything from "<storm>" to "</storm>".
  */
  void WriteStormParameters(std::fstream &oOut);

  /**
  * Writes common portions of the test error parameter files (tree pop, plot,
  * etc).
  * @param oOut File stream to write to.
  */
  void WriteCommonErrorStuff(std::fstream &oOut);

  /**
  * Writes common portions of the test parameter files for Run 1.
  * Picks up at the behavior list through the end of the file.
  * @param oOut File stream to write to.
  */
  void WriteRun1CommonStuff(std::fstream &oOut);

  /**
  * Writes common portions of the test parameter files for Run 2.
  * Picks up at the behavior list through the end of the file.
  * @param oOut File stream to write to.
  */
  void WriteRun2CommonStuff(std::fstream &oOut);

  /**
  * Writes common portions of the test parameter files for Run 3.
  * Picks up at the behavior list through the end of the file.
  * @param oOut File stream to write to.
  */
  void WriteRun3CommonStuff(std::fstream &oOut);

  /**
  * Writes common portions of the test parameter files for Run 4.
  * Picks up at the behavior list through the end of the file.
  * @param oOut File stream to write to.
  */
  void WriteRun4CommonStuff(std::fstream &oOut);

  /**
  * Writes common portions of the test parameter files for Run 4.
  * Picks up at the behavior list through the end of the file.
  * @param oOut File stream to write to.
  */
  void WriteRun5CommonStuff(std::fstream &oOut);


};

//---------------------------------------------------------------------------
#endif // TestPuertoRicoNCIMort_H
