//---------------------------------------------------------------------------
// TestNZEstablishment
//---------------------------------------------------------------------------
#if !defined(TestNZEstablishment_H)
  #define TestNZEstablishment_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>
#include <cppunit/TestSuite.h>
#include "NZEstablishment.h"

class clSimManager;

class clTestNZEstablishment : public CppUnit::TestFixture {

  public:

  /**
  * Collection of other tests.
  */
  void TestNZEstablishment();

  protected:

  /**
  * Tests clNZEstablishment::CalculateLightFavorabilities().  WriteXMLFile2()
  * writes the file that does setup.  Then a set of GLIs is placed in
  * "Establishment Light" as though GetGLIs had done it.  The expected results
  * are checked.
  * @param p_oSimManager Sim manager object.
  */
  void TestCalculateLightFavorabilities(clSimManager *p_oSimManager);

  /**
  * This test is performed by comparing the GLI calculations made by
  * clNZEstablishment by what should be an identical set of calculations made
  * by clQuadratGLILight.  Setup is performed by the file resulting from
  * WriteXMLFile1().  The clNZEstablishmentTester class will be used to check
  * the results from GetGLIs().
  * @param p_oSimManager Sim manager object.
  */
  void TestGetGLIs(clSimManager *p_oSimManager);

  /**
  * Tests clNZEstablishment::ValidateData().  WriteXMLErrorFile1 through
  * WriteXMLErrorFile4 are written to see if the expected errors are triggered.
  * @param p_oSimManager Sim manager object.
  */
  void TestValidateData(clSimManager *p_oSimManager);

 /**Tests the functions clNZEstablishment::SetFreshLogZAndRespite,
  * SetMoundZAndRespite, and SetGroundZAndRespite.  It passes in a seedling to
  * each to see if the expected results are correct for the last two (greater
  * than 0 for z and 0 for respite in the first case, 0 for both z and respite
  * in the second case).  Then 1000 seeds of species 2 are added to cell (0,0)
  * and the substrate set to all fresh logs.  The fresh log proportions array
  * will be split evenly between the four possible ages: 0, 2, 4, and 6.  Then
  * the seedlings are checked to make sure they're roughly equal to those ages.
  * @param p_oSimManager Sim manager object.
  */
  void TestSetZAndRespite(clSimManager *p_oSimManager);

  /**
  * Verifies that seeds are correctly distributed amongst the substrate types.
  * This is hard to do very well, since substrate type is not preserved.  So
  * we will set up a cell with all mineral soil and verify that half of the
  * seeds end up with z values greater than 0 and half with z values of 0.
  * WriteXMLFile4() is used for setup.
  * @param p_oSimManager Sim manager object.
  */
  void TestAction(clSimManager *p_oSimManager);

  /**
  * Produces 1000 seeds in cell 0,0, and lets them all germinate (by setting
  * substrate favorabilities to 1).  WriteXMLFile3() gives us the setup file,
  * with mound proportion = 1.  The substrate type is set to all moss (so fresh
  * logs don't interfere).  Then the z's of the seedlings will be collected and
  * written to MoundZ.txt.
  * @param p_oSimManager Sim manager object.
  */
  void TestMoundZDistributions(clSimManager *p_oSimManager);

  /**
  * Tests the function clNZEstablishment::SetFreshLogZAndRespite, to make
  * sure that seedling z heights are properly distributed.  This places 1000
  * cells in cell 0,0, and lets them all germinate.  The substrate type is set
  * to all fresh log.  WriteXMLFile3() is used to set up.  Then the z's of the
  * seedlings are collected and written to FreshLogZ.txt.
  */
  void TestFreshLogZDistributions(clSimManager *p_oSimManager);

  /**
  * Tests the function clNZEstablishment::LightTest.  WriteXMLFile2() produces
  * the setup file.  1000 seeds of species 2 are placed in cell 0,0.  The GLI
  * should be 99.989, making a light favorability of 0.8229253.  Thus we will
  * check to make sure that approximately that fraction of the seeds germinate.
  * @param p_oSimManager Sim manager object.
  */
  void TestLightTest(clSimManager *p_oSimManager);

  /**
  * Tests the function clNZEstablishment::DoesSeedSurviveSubstrate.
  * WriteXMLFile6() is used for setup.  1000 of each of the 10 species is cast.
  * The substrate is evenly divided so that each substrate will have 10% of the
  * cell.  Since each species can germinate on only one type of substrate, the
  * amount of expected seedlings of that species is 100 (10% of 1000) times the
  * favorability on the chosen substrate, plus or minus 2%.
  */
  void TestDoesSeedSurviveSubstrate(clSimManager *p_oSimManager);

  /**
  * Tests the function clNZEstablishment::GetSubstrateProportions.  There are
  * three setup files:  WriteXMLFile2(), WriteXMLFile3(), and WriteXMLFile5().
  * For all three, specific substrate proportions are assigned to cells and
  * the results compared.
  * @param p_oSimManager Sim manager object.
  */
  void TestGetSubstrateProportions(clSimManager *p_oSimManager);

  /**
  * Tests the function clNZEstablishment::GetFreshLogProportions.   There
  * are three setup files:  WriteXMLFile2(), WriteXMLFile3(), and
  * WriteXMLFile5().  For all three, specific substrate proportions are assigned
  * to cells and the results compared.
  * @param p_oSimManager Sim manager object.
  */
  void TestGetFreshLogProportions(clSimManager *p_oSimManager);

  /**
  * Writes a parameter file that sets up the quadrat light behavior and
  * NZ Establishment to have identical light parameters.  Both will calculate
  * a GLI at a height of 1 m, and have 10 by 10 meter cells.  A tree map will
  * provide a range of trees for shading.
  * @return Filename written.
  */
  char * WriteXMLFile1();

  /**
  * Writes a parameter file that sets up 6 species, 3 of which get
  * establishment.  All favorabilities are set to 1.
  * @return Filename written.
  */
  char * WriteXMLFile2();

  /**
  * Writes a parameter file that sets up 1 species.  All favorabilities are set
  * to 1.  Mound proportion is set to 1.  Mound mean height is 3 m.  Mound
  * standard deviation is 0.5 m.
  * @return Filename written.
  */
  char * WriteXMLFile3();

  /**
  * Writes a parameter file that sets up 6 species, 3 of which get
  * establishment.  All favorabilities are set to 1.  Establishment is without
  * light.
  * @return Filename written.
  */
  char * WriteXMLFile4();

  /**
   * Writes a parameter file that sets up 6 species, 3 of which get
   * establishment.  All favorabilities are set to 1.  Mound proportion is set
   * to 0.
   * @return Filename written.
   */
   char * WriteXMLFile5();

   /**
   * Writes a parameter file that sets up 10 species.  For each species, it
   * will have a favorability above 0 for a different substrate, and 0 for all
   * others.  Mound substrate is set to 0.5.
   * @return Filename written.
   */
   char * WriteXMLFile6();

  /**
  * Writes a parameter file that contains a substrate favorability that is
  * greater than 1.
  * @return Filename written.
  */
  char * WriteXMLErrorFile1();

   /**
  * Writes a parameter file that contains a mound proportion that is
  * negative.
  * @return Filename written.
  */
  char * WriteXMLErrorFile2();

  /**
  * Writes a parameter file that has light, but the light height is negative.
  * @return Filename written.
  */
  char * WriteXMLErrorFile3();

  /**
  * Writes a parameter file that has light, but the optimum GLI is greater than
  * 100.
  * @return Filename written.
  */
  char * WriteXMLErrorFile4();

};
//---------------------------------------------------------------------------
#endif // TestNZEstablishment_H
