//---------------------------------------------------------------------------
// TestSubstrate
//---------------------------------------------------------------------------
#if !defined(TestNZSubstrate_H)
  #define TestNZSubstrate_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>
#include <cppunit/TestSuite.h>

class clGridBase;
class clSimManager;

/**
* This class tests New Zealand substrate.  It depends on #define SubstrateTest
* being somewhere.
*/
class clTestNZSubstrate : public CppUnit::TestFixture
{

  public:

  /**
  * Rolls up the NZ Substrate testing.
  */
  void TestNZSubstrate();

  /**
  * As a visual check - this produces output of the substrate curves so they
  * can be double-checked.  This uses WriteXMLFile4(), WriteXMLFile5(), and
  * WriteXMLFile6() and produces a file called "Substrate.txt".  It produces a
  * constant input of fresh logs to grid cell (0,0) of 0.01 (as a proportion).
  * *File5 produces a low mossy log decay rate; *File6 a medium one; and *File7
  * a high rate.  Then it outputs fresh and mossy logs for the three rates.
  */
  void ProduceDecayGraphs();

  protected:

  /**
  * Tests snag additions to fresh log.  This will create snags and set them
  * as dead.  They should all be added to fresh log, despite having the
  * proportion of dead that fall set to 0.  WriteXMLFile10() is used for setup.
  */
  void TestSnagSubstrate(clSimManager *p_oSimManager);

  /**
  * Tests tree fall.  This makes sure that fallen trees are removed, and
  * standing trees have the opportunity to become snags.  This uses
  * WriteXMLFile8() to do setup.  It has three species:  for species 1, 20% of
  * dead trees fall; for species 2, 50%; and for species 3, 0%.  There is
  * a behavior which is applied to snags, to ensure standing dead trees produce
  * snags.  1000 trees are created for each species, and all are set to dead.
  * Then one timestep is run, and the snags counted to make sure there is the
  * proper proportion of each.  Then another timestep is run to make sure the
  * snags aren't "disappeared".  This process is repeated for WriteXMLFile9(),
  * which has a longer timestep length.
  */
  void TestTreeFall(clSimManager *p_oSimManager);

  /**
  * Tests aspects of substrate grid setup.  This will enter two parameter
  * files: one with grid maps and one without maps.  In the first case, grid
  * cell resolution should match what was specified in the map; in the second,
  * it should match the plot.  This will also make sure that initial conditions
  * have been correctly populated.
  */
  void TestSubstrateGridSetup(clSimManager *p_oSimManager);

  /**
  * Verifies all aspects of substrate.  Tests that new fresh log area and new
  * mineral soil area are added correctly when trees die, that cohorts are
  * managed correctly, and that the final substrate numbers are calculated
  * correctly.  WriteXMLFile4() is used to set up a grid with trees in known
  * locations with fall and tip-up probabilities of 1.  This executes the
  * timesteps and provides mortality (killing known trees at certain times), and
  * then verifies grid values.
  */
  void TestSubstrateProcessing(clSimManager *p_oSimManager);

  /**
  * Verifies the substrate proportions for timestep 1 of TestSubstrate().
  */
  void VerifyTimestep1SubstrateProportions(clGridBase *p_oSubstrateGrid);

  /**
  * Verifies the substrate proportions for timestep 2 of TestSubstrate().
  */
  void VerifyTimestep2SubstrateProportions(clGridBase *p_oSubstrateGrid);

  /**
  * Verifies the substrate proportions for timestep 3 of TestSubstrate().
  */
  void VerifyTimestep3SubstrateProportions(clGridBase *p_oSubstrateGrid);

  /**
  * Verifies the substrate proportions for timestep 4 of TestSubstrate().
  */
  void VerifyTimestep4SubstrateProportions(clGridBase *p_oSubstrateGrid);

  /**
  * Verifies the cohort structure for timestep 1 of TestSubstrate().
  */
  void VerifyTimestep1Cohorts(clGridBase *p_oSubstrateGrid);

  /**
  * Verifies the cohort structure for timestep 2 of TestSubstrate().
  */
  void VerifyTimestep2Cohorts(clGridBase *p_oSubstrateGrid);

  /**
  * Verifies the cohort structure for timestep 3 of TestSubstrate().
  */
  void VerifyTimestep3Cohorts(clGridBase *p_oSubstrateGrid);

  /**
  * Verifies the cohort structure for timestep 4 of TestSubstrate().
  */
  void VerifyTimestep4Cohorts(clGridBase *p_oSubstrateGrid);

  /**
  * Verifies new organics for timestep 1 of TestSubstrate().
  */
  void VerifyTimestep1MineralSoil(clGridBase *p_oCalcsGrid);

  /**
  * Verifies new organics for timestep 2 of TestSubstrate().
  */
  void VerifyTimestep2MineralSoil(clGridBase *p_oCalcsGrid);

  /**
  * Verifies new organics for timestep 3 of TestSubstrate().
  */
  void VerifyTimestep3MineralSoil(clGridBase *p_oCalcsGrid);

  /**
  * Verifies new organics for timestep 4 of TestSubstrate().
  */
  void VerifyTimestep4MineralSoil(clGridBase *p_oCalcsGrid);

  /**
  * Verifies the fresh logs arrays for timestep 1 of TestSubstrate().
  */
  void VerifyTimestep1FreshLogArray(clGridBase *p_oCalcsGrid);
  /**
  * Verifies the fresh logs arrays for timestep 2 of TestSubstrate().
  * Individual cells can't be verified but total for the grid can be.
  */
  void VerifyTimestep2FreshLogArray(clGridBase *p_oCalcsGrid);

/**
  * Verifies the fresh logs arrays for timestep 3 of TestSubstrate().
  * Individual cells can't be verified but total for the grid can be.
  */
  void VerifyTimestep3FreshLogArray(clGridBase *p_oCalcsGrid);

/**
  * Verifies the fresh logs arrays for timestep 4 of TestSubstrate().
  * Individual cells can't be verified but total for the grid can be.
  */
  void VerifyTimestep4FreshLogArray(clGridBase *p_oCalcsGrid);

  /**
  * Writes a parameter file with a Substrate grid
  * map, for testing the grid cell resolution of "NZ Substrate".
  * @return Filename written.
  */
  char * WriteXMLFile1();

  /**
  * Writes a parameter file with no Substrate grid
  * map, for testing the grid cell resolution of "NZ Substrate".
  * @return Filename written.
  */
  char * WriteXMLFile2();

  /**
  * Writes a parameter file for testing tree mortality.  Fall and tip-up
  * probabilities of 1 to guarantee new fresh logs and new organics.
  */
  char * WriteXMLFile3();

  /**
  * Writes a parameter file that allows for testing of substrate curves.  This
  * has a low value for mossy log decay.
  */
  char * WriteXMLFile4();

  /**
  * Writes a parameter file that allows for testing of substrate curves.  This
  * has a medium value for mossy log decay.
  */
  char * WriteXMLFile5();

  /**
  * Writes a parameter file that allows for testing of substrate curves.  This
  * has a high value for mossy log decay.
  */
  char * WriteXMLFile6();

  /**
  * Writes a parameter file for testing tree fall.  There are three species.
  * For species 1, 20% of dead trees fall; for species 2, 50%; and for species
  * 3, 0%.  There is a behavior which is applied to snags.  The timestep length
  * is 1 year.
  * @return Filename written.
  */
  char * WriteXMLFile8();

  /**
  * Writes a parameter file for testing tree fall.  There are three species.
  * For species 1, 20% of dead trees fall; for species 2, 50%; and for species
  * 3, 0%.  There is a behavior which is applied to snags.  The timestep length
  * is 3 years.
  * @return Filename written.
  */
  char * WriteXMLFile9();

  /**
  * Writes a parameter file for testing snag additions to substrate.  There is
  * one species.  The proportion of dead that fall is 0.  The proportion of
  * fallen that uproot is 1.
  * @return Filename written.
  */
  char * WriteXMLFile10();
};
//---------------------------------------------------------------------------
#endif // TestSubstrate
