//---------------------------------------------------------------------------
// TestSubstrate
//---------------------------------------------------------------------------
#if !defined(TestSubstrate_H)
  #define TestSubstrate_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>
#include <cppunit/TestSuite.h>

class clGridBase;
class clSimManager;

/**
* This class tests substrate.
*/
class clTestSubstrate : public CppUnit::TestFixture
{

  public:

  /**
  * As a visual check - this produces output of the substrate curves so they
  * can be double-checked.  This uses WriteXMLFile5(), WriteXMLFile6(), and
  * WriteXMLFile7() and produces a file called "Substrate.txt".  It produces a
  * constant input of fresh logs to grid cell (0,0) of 0.01 (as a proportion).
  * *File5 produces a low mossy log decay rate; *File6 a medium one; and *File7
  * a high rate.  Then it outputs fresh and mossy logs for the three rates.
  */
  void ProduceDecayGraphs();

  /**
  * Groups the other substrate tests.
  */
  void TestSubstrate();

  protected:

  /**
  * Tests snag additions to fresh log.  This will create snags and set them
  * as dead.  They should all be added to fresh log, despite having the
  * proportion of dead that fall set to 0.  WriteXMLFile10() is used for setup.
  */
  void TestSnagSubstrate(clSimManager *p_oSimManager);

  /**
  * Tests tree fall.  This makes sure that fallen trees are removed, and
  * standing trees have the opportunity to become snags.  This uses
  * WriteXMLFile8() to do setup.  It has three species:  for species 1, 20% of
  * dead trees fall; for species 2, 50%; and for species 3, 0%.  There is
  * a behavior which is applied to snags, to ensure standing dead trees produce
  * snags.  1000 trees are created for each species, and all are set to dead.
  * Then one timestep is run, and the snags counted to make sure there is the
  * proper proportion of each.  Then another timestep is run to make sure the
  * snags aren't "disappeared".  This process is repeated for WriteXMLFile9(),
  * which has a longer timestep length.
  */
  void TestTreeFall(clSimManager *p_oSimManager);

  /**
  * Tests aspects of substrate grid setup.  This will enter three parameter
  * files:
  * <ol>
  * <li>With harvest
  * <li>With no harvest and Substrate and substratecalcs grid maps
  * <li>With neither
  * </ol>
  * In the first case, grid cell resolution should match harvest;
  * in the second, it should be what was specified in the map; in the third,
  * it should match the plot.  This will also make sure that initial conditions
  * have been correctly populated.
  */
  void TestSubstrateGridSetup(clSimManager *p_oSimManager);

  /**
  * Verifies that harvest substrate is calculated correctly.  WriteXMLFile1()
  * produces all cut types in various grid cells; in those cells, the substrate
  * proportions should match the harvest conditions, and there should be only
  * one cohort with the harvest's conditions.
  */
  void TestHarvestSubstrate(clSimManager *p_oSimManager);

  /**
  * Verifies all aspects of substrate.  Tests that new fresh log area and new
  * organics area are added correctly when trees die, that cohorts are managed
  * correctly, and that the final substrate numbers are calculated correctly.
  * WriteXMLFile4() is used to set up a grid with trees in known locations with
  * fall and tip-up probabilities of 1.  This also has harvests occuring in
  * three grid cells so harvest cohorts can be tested.  This executes the
  * timesteps and provides mortality (killing known trees at certain times), and
  * then verifies grid values.
  */
  void TestSubstrateCalcs(clSimManager *p_oSimManager);

  /**
  * Verifies the substrate proportions for timestep 1 of TestSubstrate().
  */
  void VerifyTimestep1SubstrateProportions(clGridBase *p_oSubstrateGrid);

  /**
  * Verifies the substrate proportions for timestep 2 of TestSubstrate().
  */
  void VerifyTimestep2SubstrateProportions(clGridBase *p_oSubstrateGrid);

  /**
  * Verifies the substrate proportions for timestep 3 of TestSubstrate().
  */
  void VerifyTimestep3SubstrateProportions(clGridBase *p_oSubstrateGrid);

  /**
  * Verifies the substrate proportions for timestep 4 of TestSubstrate().
  */
  void VerifyTimestep4SubstrateProportions(clGridBase *p_oSubstrateGrid);

  /**
  * Verifies the cohort structure for timestep 1 of TestSubstrate().
  */
  void VerifyTimestep1Cohorts(clGridBase *p_oSubstrateGrid);

  /**
  * Verifies the cohort structure for timestep 2 of TestSubstrate().
  */
  void VerifyTimestep2Cohorts(clGridBase *p_oSubstrateGrid);

  /**
  * Verifies the cohort structure for timestep 3 of TestSubstrate().
  */
  void VerifyTimestep3Cohorts(clGridBase *p_oSubstrateGrid);

  /**
  * Verifies the cohort structure for timestep 4 of TestSubstrate().
  */
  void VerifyTimestep4Cohorts(clGridBase *p_oSubstrateGrid);

  /**
  * Verifies new organics for timestep 1 of TestSubstrate().
  */
  void VerifyTimestep1Organics(clGridBase *p_oCalcsGrid);

  /**
  * Verifies new organics for timestep 2 of TestSubstrate().
  */
  void VerifyTimestep2Organics(clGridBase *p_oCalcsGrid);

  /**
  * Verifies new organics for timestep 3 of TestSubstrate().
  */
  void VerifyTimestep3Organics(clGridBase *p_oCalcsGrid);

  /**
  * Verifies new organics for timestep 4 of TestSubstrate().
  */
  void VerifyTimestep4Organics(clGridBase *p_oCalcsGrid);

  /**
  * Verifies the fresh logs arrays for timestep 1 of TestSubstrate().
  */
  void VerifyTimestep1FreshLogArray(clGridBase *p_oCalcsGrid);
  /**
  * Verifies the fresh logs arrays for timestep 1 of TestSubstrate().
  */
  void VerifyTimestep2FreshLogArray(clGridBase *p_oCalcsGrid);

/**
  * Verifies the fresh logs arrays for timestep 1 of TestSubstrate().
  */
  void VerifyTimestep3FreshLogArray(clGridBase *p_oCalcsGrid);

/**
  * Verifies the fresh logs arrays for timestep 1 of TestSubstrate().
  */
  void VerifyTimestep4FreshLogArray(clGridBase *p_oCalcsGrid);


 /**
  * Writes a parameter file with harvest defined and a tree map with all
  * possible harvest types.  For testing:
  * <ul>
  * <li>the grid cell resolution of "Substrate" should match Harvest
  * <li>substrate conditions after a harvest should match the defined harvest
  * conditions exactly, with one cohort with the harvest conditions
  * </ul>
  * @return Filename written.
  */
  char * WriteXMLFile1();

  /**
  * Writes a parameter file with no harvest defined but with Substrate and
  * substratecalcs grid maps, for testing the grid cell resolution of
  * "Substrate".
  * @return Filename written.
  */
  char * WriteXMLFile2();

  /**
  * Writes a parameter file with no harvest defined and no Substrate grid
  * map, for testing the grid cell resolution of "Substrate".
  * @return Filename written.
  */
  char * WriteXMLFile3();

  /**
  * Writes a parameter file for testing tree mortality.  It contains a tree map
  * with various-size trees, and fall and tip-up probabilities of 1 to guarantee
  * new fresh logs and new organics.
  * @return Filename written.
  */
  char * WriteXMLFile4();

  /**
  * Writes a parameter file that allows for testing of substrate curves.  This
  * has a low value for mossy log decay.
  * @return Filename written.
  */
  char * WriteXMLFile5();

  /**
  * Writes a parameter file that allows for testing of substrate curves.  This
  * has a medium value for mossy log decay.
  * @return Filename written.
  */
  char * WriteXMLFile6();

  /**
  * Writes a parameter file that allows for testing of substrate curves.  This
  * has a high value for mossy log decay.
  * @return Filename written.
  */
  char * WriteXMLFile7();

  /**
  * Writes a parameter file for testing tree fall.  There are three species.
  * For species 1, 20% of dead trees fall; for species 2, 50%; and for species
  * 3, 0%.  There is a behavior which is applied to snags.  The timestep length
  * is 1 year.
  * @return Filename written.
  */
  char * WriteXMLFile8();

  /**
  * Writes a parameter file for testing tree fall.  There are three species.
  * For species 1, 20% of dead trees fall; for species 2, 50%; and for species
  * 3, 0%.  There is a behavior which is applied to snags.  The timestep length
  * is 3 years.
  * @return Filename written.
  */
  char * WriteXMLFile9();

  /**
  * Writes a parameter file for testing snag additions to substrate.  There is
  * one species.  The proportion of dead that fall is 0.  The proportion of
  * fallen that uproot is 1.
  * @return Filename written.
  */
  char * WriteXMLFile10();

};
//---------------------------------------------------------------------------
#endif // TestSubstrate
